#!/usr/bin/perl
use lib '/home/topeg/PERL/AA_LIBS';
use strict;
use warnings;
use Linux::Disks;
use Data::Dumper;
use Gtk2 -init;
use FindBin '$Bin';

my %icons=(
  ata  => $Bin.'/icons/ata.svg',
  usb  => $Bin.'/icons/usb.svg',
  scsi => $Bin.'/icons/scsi.svg',
  net  => $Bin.'/icons/net.svg',
  raid => $Bin.'/icons/raid.svg',
);

#my $data=eval{local($/,@ARGV)=(undef,$Bin.'/test.txt'); <>};
#$data=eval("my $data;return \$VAR1;");
#my $disks=Linux::Disks->new(fake => $data);

my $disks=Linux::Disks->new();

#print Dumper($disks);

my $gui=GUI->new("$Bin/gui.glade");
$gui->signal_connect(do_exit => sub{ Gtk2->main_quit; });

my %partitions=();
runn($gui,$disks);

$gui->signal_connect(do_update => sub{
    $disks->update();
    #for my $part ($disks->partitions())
    #{
    #  my $fs=$part->fs_type();
    #  next unless($fs && $fs ne 'swap');

    #  my $gui_part=$partitions{$part->name()};
    #  $gui_part->mount_point($part->mount_point());
    #  $gui_part->size($part->size());
    #  $gui_part->mounted($part->mounted());
    #  $gui_part->fs_type($fs);
    #}
    runn($gui,$disks);
  });

Gtk2->main;

########################################################################
sub runn
{
  my $gui=shift;
  my $disks=shift;
  $gui->reset_tabs();
  summary($gui,$disks);
  entrys($gui,$disks);
  $gui->show_all();
}

sub entrys
{
  my $gui=shift;
  my $disks=shift;

  %partitions=();
  #$gui->reset_tabs();
  for my $dev ($disks->drives())
  {

    next if($dev->mountable() && $dev->special_fs());

    my $icon;

    my $btype=lc($dev->bus_type());
    if(exists($icons{$btype}))
    { $icon=$icons{$btype}; }

    my $dname=$dev->vendor().' '.$dev->model();
    $dname=~s/^\s+//s;
    $dname=~s/\s+$//s;

    my $tab=$gui->add_disk_tab($dev->name(),uc($btype),$dname,$icon);

    if($dev->mountable())
    {
      my @l=(
          $dev->label(),
          $dev->name(),
          $dev->mount_point(),
          $dev->size(),
          $dev->free(),
          $dev->mounted(),
          $dev->fs_type(),
          $dev->percent(),
        );
      my $entry=$tab->add_entry(@l);
      $partitions{$dev->name()}=$entry;
    }
    else
    {
      for my $part ($dev->partitions())
      {
        next if($part->special_fs());
        my @l=(
            $part->label(),
            $part->name(),
            $part->mount_point(),
            $part->size(),
            $part->free(),
            $part->mounted(),
            $part->fs_type(),
            $part->percent(),
          );
        my $entry=$tab->add_entry(@l);
        $partitions{$part->name()}=$entry;
      }
    }
  }
}

sub summary
{
  my $gui=shift;
  my $disks=shift;

  my $tab=$gui->add_disk_tab('Summary','INFO','INFO','gtk-about');

  for my $dev ($disks->drives())
  {
    next if($dev->mountable() && $dev->special_fs());
    if($dev->mountable())
    {
      my @l=(
          $dev->label(),
          $dev->name(),
          $dev->mount_point(),
          $dev->size(),
          $dev->free(),
          $dev->mounted(),
          $dev->fs_type(),
          $dev->percent(),
        );
      $tab->add_entry(@l);
    }
    else
    {
      for my $part ($dev->partitions())
      {
        next if($part->special_fs());
        my @l=(
            $part->label(),
            $part->name(),
            $part->mount_point(),
            $part->size(),
            $part->free(),
            $part->mounted(),
            $part->fs_type(),
            $part->percent(),
          );
        $tab->add_entry(@l);
      }
    }
  }
}


########################################################################
########################################################################

package GUI;
use Gtk2;

use Glib::Object::Subclass
  Gtk2::Builder::,
  signals => {
    do_exit => {
      flags => ['run-last'],
      param_types => [ ],
    },
    do_info => {
      flags => ['run-last'],
      param_types => [ ],
    },
    do_update => {
      flags => ['run-last'],
      param_types => [ ],
    },
  },
  properties => [];

sub new
{
  my $class=shift;
  my $file=shift;
  my $self=$class->SUPER::new();

  if(defined($file))
  {
    my $ff=eval{local ($/,@ARGV)=(undef,$file); <>; };
    return undef unless($self->add_from_string($ff));
  }
  #else
  #{ return undef unless($self->add_from_string(eval{local $/=undef; <DATA> })); }


  $self->reset_tabs();

  $self->connect_signals($self);
  return $self;
}

sub reset_tabs
{
  my $self=shift;
  my $tab=$self->disk_tabs();
  while($tab->get_n_pages())
  {
    my $id=$tab->get_current_page();
    $tab->remove_page($id);
  }
}

sub mainwin   { return $_[0]->get_object('mainwin'); }
sub disk_tabs { return $_[0]->get_object('disk_tabs'); }

sub add_disk_tab
{
  my $self=shift;

  my $intern=GUI::Disk->new(@_);

  $self->disk_tabs()->append_page_menu( $intern->_widgets() );
  return $intern;
}

#-----------------------------------------------------------------------
sub on_exit
{
  my $self=pop;
  $self->mainwin()->destroy();
  $self->signal_emit('do_exit');
  return 1;
}


sub on_info
{
  my $self=pop;
  $self->signal_emit('do_info');
}

sub on_update
{
  my $self=pop;
  $self->signal_emit('do_update');
}

#-----------------------------------------------------------------------
sub show_all{ $_[0]->mainwin()->show_all(); }

########################################################################

package GUI::Disk;
use Gtk2;
use Gtk2::Ex::Simple::List;

sub BEGIN
{
  my @sizes=qw(Byte Kib Mib Gib Tib);
  Gtk2::Ex::Simple::List->add_column_type( 'fs_size',
                     type     => 'Glib::Scalar',
                     renderer => 'Gtk2::CellRendererText',
                     attr     => sub {
                          my ($treecol, $cell, $model, $iter, $col_num) = @_;
                          my $size = $model->get ($iter, $col_num);
                          my $t='';
                          for(@sizes)
                          {
                            $t=$_;
                            last if($size<1024);
                            $size/=1024;
                          }
                          $cell->set (text => sprintf('%.02f %s',$size,$t));
                     }
                );


  Gtk2::Ex::Simple::List->add_column_type( 'fs_mount',
                     type     => 'Glib::Scalar',
                     renderer => 'Gtk2::CellRendererPixbuf',
                     attr     => sub {
                          my ($treecol, $cell, $model, $iter, $col_num) = @_;
                          my $ok = $model->get ($iter, $col_num);
                          if($ok)
                          { $cell->set ('stock-id' => 'gtk-yes'); }
                          else
                          { $cell->set ('stock-id' => 'gtk-no'); }
                     }
                );

  Gtk2::Ex::Simple::List->add_column_type( 'fs_fill',
                     type     => 'Glib::Scalar',
                     renderer => 'Gtk2::CellRendererProgress',
                     attr     => sub {
                          my ($treecol, $cell, $model, $iter, $col_num) = @_;
                          my $percent = $model->get ($iter, $col_num);
                          $cell->set('orientation' => 'left-to-right');
                          $cell->set('text' => "$percent%");
                          $cell->set('text-xalign' => 0.5);
                          $cell->set('text-yalign' => 0.5);
                          $cell->set ('value' => $percent);
                     }
                );
}

sub new
{
  my $class=shift;
  my $name=shift // '';
  my $bus_name=shift // '';
  my $dev_name=shift // '';
  my $icon=shift;

  #print "ICON: $icon\n" if($icon);

  if($icon)
  {
    if(-f $icon)
    {
      my $pixbuf=Gtk2::Gdk::Pixbuf->new_from_file_at_scale($icon, 16, 16, 1);
      $icon=Gtk2::Image->new_from_pixbuf($pixbuf);
    }
    else
    { $icon=Gtk2::Image->new_from_icon_name ($icon, 'menu'); }
  }
  else
  { $icon=Gtk2::Image->new_from_icon_name ('gtk-harddisk', 'menu'); }

  my $self={name=>$name};

  my $box=Gtk2::HBox->new(0,5);
  $box->pack_start($icon, 1, 1, 0);
  $box->pack_start(Gtk2::Label->new($name), 1, 1, 0);
  $box->show_all();
  $self->{title}=$box;

  $box=Gtk2::VBox->new(0,5);

  my $table=Gtk2::Table->new (2, 2, 0);
  $box->pack_start($table, 0, 1, 0);

  my $l=Gtk2::Label->new('Bus Type:');
  $l->set('xalign',1);
  $table->attach($l, 0, 1, 0, 1, 'fill', 'fill', 5, 5);

  $l=Gtk2::Label->new('Device Name:');
  $l->set('xalign',1);
  $table->attach($l, 0, 1, 1, 2, 'fill', 'fill', 5, 5);

  $l=Gtk2::Label->new($bus_name);
  $table->attach($l, 1, 2, 0, 1, 'fill', 'fill', 5, 5);
  $l->set('xalign',0);

  $l=Gtk2::Label->new($dev_name);
  $table->attach($l, 1, 2, 1, 2, 'fill', 'fill', 5, 5);
  $l->set('xalign',0);

  #$table->show_all();

  my $scrolled_window = Gtk2::ScrolledWindow->new;
  $box->pack_start($scrolled_window, 1, 1, 0);
  $scrolled_window->set_policy ('automatic', 'automatic');
  $scrolled_window->set_shadow_type ('in');
  my $slist = Gtk2::Ex::Simple::List->new (
                       'Name'     => 'text',
                       'Device'   => 'text',
                       'Mount'    => 'text',
                       'Size'     => 'fs_size',
                       'Free'     => 'fs_size',
                       'Mounted'  => 'fs_mount',
                       'FS-Type'  => 'text',
                       'Usage'    => 'fs_fill',
                     );
  $scrolled_window->add($slist);
  $self->{list}=$slist;
  $self->{win}=$box;

  bless($self,$class);
}

sub _widgets
{
  my $self=shift;
  return ($self->{win},$self->{title},undef);
}

sub name
{
  my $self=shift;
  if(@_)
  { $self->{label}->set_label(@_); }
  return $self->{label}->get_label();
}

sub add_entry
{
  my $self=shift;
  my $entry=GUI::Disk::Part->new($self->{list}->{data},@_);
  return $entry;
}

########################################################################

package GUI::Disk::Part;

sub new
{
  my $class=shift;
  my $elm=shift // [];
  my $ndev=shift // '';
  my $name=shift // '';
  my $mount=shift // '';
  my $size=shift // 0;
  my $free=shift // 0;
  my $mounted=shift // 0;
  my $fs=shift // '';
  my $usage=shift // 0;

  push(@$elm,[$ndev,$name,$mount,$size,$free,$mounted,$fs,$usage]);

  my $self={ elm=>$elm, lst_pos=>$#{$elm}, pos=>{
      name  =>0,
      dev   =>1,
      mount =>2,
      size  =>3,
      free  =>4,
      fsmnt =>5,
      fs    =>6,
      usage =>7,
    } };

  return bless($self,$class);
}

sub _pos
{
  my $self=shift;
  return $self->{pos}->{$_[0]};
}

sub list_pos
{
  my $self=shift;
  $self->{lst_pos}=$_[0] if(@_);
  return $self->{lst_pos};
}

sub _val
{
  my $self=shift;
  my $name=shift;
  my $p=$self->_pos($name);
  $self->{elm}->[$self->list_pos()]->[$p]=$_[0] if(@_);
  return $self->{elm}->[$self->list_pos()]->[$p];
}

sub name        { return $_[0]->_val('name'); }
sub fs_type     { return $_[0]->_val('fs'); }
sub size        { return $_[0]->_val('size'); }
sub mount_point { return $_[0]->_val('mount'); }
sub mounted     { return $_[0]->_val('fsmnt'); }
sub usage       { return $_[0]->_val('usage'); }
sub device      { return $_[0]->_val('dev'); }
sub free        { return $_[0]->_val('free'); }

1;